@@ -1,6 +1,7 @@
-use super::{ModuleError, ModuleValidator};
 use parity_wasm::elements::Module;
 
+use super::{ChiselModule, ModuleError, ModuleKind, ModuleValidator};
+
 /// Struct on which ModuleValidator is implemented.
 pub struct CheckStartFunc {
     start_required: bool,
@@ -14,6 +15,22 @@ impl CheckStartFunc {
     }
 }
 
+impl<'a> ChiselModule<'a> for CheckStartFunc {
+    type ObjectReference = &'a dyn ModuleValidator;
+
+    fn id(&'a self) -> String {
+        "checkstartfunc".to_string()
+    }
+
+    fn kind(&'a self) -> ModuleKind {
+        ModuleKind::Validator
+    }
+
+    fn as_abstract(&'a self) -> Self::ObjectReference {
+        self as Self::ObjectReference
+    }
+}
+
 impl ModuleValidator for CheckStartFunc {
     fn validate(&self, module: &Module) -> Result<bool, ModuleError> {
         Ok(module.start_section().is_some() == self.start_required)
@@ -23,7 +40,6 @@ impl ModuleValidator for CheckStartFunc {
 #[cfg(test)]
 mod tests {
     use super::*;
-    use parity_wasm::elements::deserialize_buffer;
 
     #[test]
     fn start_required_good() {
@@ -33,7 +49,7 @@ mod tests {
             0x08, 0x01, 0x00, 0x0a, 0x04, 0x01, 0x02, 0x00, 0x0b,
         ];
 
-        let module = deserialize_buffer::<Module>(&wasm).unwrap();
+        let module = Module::from_bytes(&wasm).unwrap();
         let checker = CheckStartFunc::new(true);
 
         let result = checker.validate(&module).unwrap();
@@ -48,7 +64,7 @@ mod tests {
             0x08, 0x01, 0x00, 0x0a, 0x04, 0x01, 0x02, 0x00, 0x0b,
         ];
 
-        let module = deserialize_buffer::<Module>(&wasm).unwrap();
+        let module = Module::from_bytes(&wasm).unwrap();
         let checker = CheckStartFunc::new(false);
 
         let result = checker.validate(&module).unwrap();
@@ -63,7 +79,7 @@ mod tests {
             0x0a, 0x04, 0x01, 0x02, 0x00, 0x0b,
         ];
 
-        let module = deserialize_buffer::<Module>(&wasm).unwrap();
+        let module = Module::from_bytes(&wasm).unwrap();
         let checker = CheckStartFunc::new(false);
 
         let result = checker.validate(&module).unwrap();
@@ -78,7 +94,7 @@ mod tests {
             0x0a, 0x04, 0x01, 0x02, 0x00, 0x0b,
         ];
 
-        let module = deserialize_buffer::<Module>(&wasm).unwrap();
+        let module = Module::from_bytes(&wasm).unwrap();
         let checker = CheckStartFunc::new(true);
 
         let result = checker.validate(&module).unwrap();
