@@ -1,23 +1,36 @@
-use super::{ModuleCreator, ModuleError};
 use parity_wasm::builder;
 use parity_wasm::elements::{CustomSection, Module};
 
-use byteorder::{LittleEndian, WriteBytesExt};
+use super::{ChiselModule, ModuleError, ModuleKind, ModulePreset, ModuleTranslator};
 
-/// Enum on which ModuleCreator is implemented.
-pub enum Deployer<'a> {
-    Memory(&'a [u8]),
-    CustomSection(&'a [u8]),
+/// Enum on which ModuleTranslator is implemented.
+pub enum Deployer {
+    Memory,
+    CustomSection,
 }
 
-// FIXME: Bring ModulePreset API in line with the other with_preset methods so a ModulePreset impl
-// can be written
-impl<'a> Deployer<'a> {
-    pub fn with_preset(preset: &str, payload: &'a [u8]) -> Result<Self, ()> {
+impl<'a> ChiselModule<'a> for Deployer {
+    type ObjectReference = &'a dyn ModuleTranslator;
+
+    fn id(&'a self) -> String {
+        "deployer".to_string()
+    }
+
+    fn kind(&'a self) -> ModuleKind {
+        ModuleKind::Translator
+    }
+
+    fn as_abstract(&'a self) -> Self::ObjectReference {
+        self as Self::ObjectReference
+    }
+}
+
+impl ModulePreset for Deployer {
+    fn with_preset(preset: &str) -> Result<Self, ModuleError> {
         match preset {
-            "memory" => Ok(Deployer::Memory(payload)),
-            "customsection" => Ok(Deployer::CustomSection(payload)),
-            _ => Err(()),
+            "memory" => Ok(Deployer::Memory),
+            "customsection" => Ok(Deployer::CustomSection),
+            _ => Err(ModuleError::NotSupported),
         }
     }
 }
@@ -71,11 +84,11 @@ fn create_custom_deployer(payload: &[u8]) -> Result<Module, ModuleError> {
     let code = deployer_code();
 
     // This is the pre-written deployer code.
-    let mut module: Module = parity_wasm::deserialize_buffer(&code)?;
+    let mut module = Module::from_bytes(&code)?;
 
     // Re-write memory to pre-allocate enough for code size
     let memory_initial = (payload.len() as u32 / 65536) + 1;
-    let mem_type = parity_wasm::elements::MemoryType::new(memory_initial, None, false);
+    let mem_type = parity_wasm::elements::MemoryType::new(memory_initial, None);
     module
         .memory_section_mut()
         // This would be an internal error (.e.g the the deployer code above has no memory section)
@@ -83,8 +96,10 @@ fn create_custom_deployer(payload: &[u8]) -> Result<Module, ModuleError> {
         .entries_mut()[0] = mem_type;
 
     // Prepare payload (append length).
+    let payload_len = payload.len() as u32;
+    let payload_len = payload_len.to_le_bytes();
     let mut custom_payload = payload.to_vec();
-    custom_payload.write_i32::<LittleEndian>(payload.len() as i32)?;
+    custom_payload.extend_from_slice(&payload_len);
 
     // Prepare and append custom section.
     let custom = CustomSection::new("deployer".to_string(), custom_payload);
@@ -155,29 +170,33 @@ fn create_memory_deployer(payload: &[u8]) -> Module {
         .build()
 }
 
-impl<'a> ModuleCreator for Deployer<'a> {
-    fn create(&self) -> Result<Module, ModuleError> {
-        let output = match self {
-            Deployer::Memory(payload) => create_memory_deployer(&payload),
-            Deployer::CustomSection(payload) => create_custom_deployer(&payload)?,
-        };
+impl ModuleTranslator for Deployer {
+    fn translate_inplace(&self, _module: &mut Module) -> Result<bool, ModuleError> {
+        Err(ModuleError::NotSupported)
+    }
 
-        Ok(output)
+    fn translate(&self, module: &Module) -> Result<Option<Module>, ModuleError> {
+        let payload = module.clone().to_bytes()?;
+        let output = match self {
+            Deployer::Memory => create_memory_deployer(&payload),
+            Deployer::CustomSection => create_custom_deployer(&payload)?,
+        };
+        Ok(Some(output))
     }
 }
 
 #[cfg(test)]
 mod tests {
-    use super::*;
-    use parity_wasm;
     use rustc_hex::FromHex;
 
+    use super::*;
+
     #[test]
     fn zero_payload() {
         let payload = vec![];
-        let module = Deployer::with_preset("customsection", &payload)
+        let output = create_custom_deployer(&payload)
             .unwrap()
-            .create()
+            .to_bytes()
             .unwrap();
         let expected = FromHex::from_hex(
             "
@@ -192,16 +211,15 @@ mod tests {
         ",
         )
         .unwrap();
-        let output = parity_wasm::serialize(module).expect("Failed to serialize");
         assert_eq!(output, expected);
     }
 
     #[test]
     fn nonzero_payload() {
         let payload = FromHex::from_hex("80ff007faa550011").unwrap();
-        let module = Deployer::with_preset("customsection", &payload)
+        let output = create_custom_deployer(&payload)
             .unwrap()
-            .create()
+            .to_bytes()
             .unwrap();
         let expected = FromHex::from_hex(
             "
@@ -216,17 +234,13 @@ mod tests {
         ",
         )
         .unwrap();
-        let output = parity_wasm::serialize(module).expect("Failed to serialize");
         assert_eq!(output, expected);
     }
 
     #[test]
     fn big_payload() {
         let payload = [0; 632232];
-        let module = Deployer::with_preset("customsection", &payload)
-            .unwrap()
-            .create()
-            .unwrap();
+        let module = create_custom_deployer(&payload).unwrap();
         let memory_initial = module.memory_section().unwrap().entries()[0]
             .limits()
             .initial();
@@ -236,10 +250,7 @@ mod tests {
     #[test]
     fn memory_zero_payload() {
         let payload = vec![];
-        let module = Deployer::with_preset("memory", &payload)
-            .unwrap()
-            .create()
-            .unwrap();
+        let output = create_memory_deployer(&payload).to_bytes().unwrap();
         let expected = FromHex::from_hex(
             "
             0061736d0100000001090260027f7f0060000002130108657468657265756d0666
@@ -248,17 +259,13 @@ mod tests {
         ",
         )
         .unwrap();
-        let output = parity_wasm::serialize(module).expect("Failed to serialize");
         assert_eq!(output, expected);
     }
 
     #[test]
     fn memory_nonzero_payload() {
         let payload = FromHex::from_hex("80ff007faa550011").unwrap();
-        let module = Deployer::with_preset("memory", &payload)
-            .unwrap()
-            .create()
-            .unwrap();
+        let output = create_memory_deployer(&payload).to_bytes().unwrap();
         let expected = FromHex::from_hex(
             "
             0061736d0100000001090260027f7f0060000002130108657468657265756d0666
@@ -268,20 +275,60 @@ mod tests {
         ",
         )
         .unwrap();
-        let output = parity_wasm::serialize(module).expect("Failed to serialize");
         assert_eq!(output, expected);
     }
 
     #[test]
     fn memory_big_payload() {
         let payload = [0; 632232];
-        let module = Deployer::with_preset("memory", &payload)
-            .unwrap()
-            .create()
-            .unwrap();
+        let module = create_memory_deployer(&payload);
         let memory_initial = module.memory_section().unwrap().entries()[0]
             .limits()
             .initial();
         assert_eq!(memory_initial, 10);
     }
+
+    #[test]
+    fn customsection_interface_test() {
+        let payload = Module::default();
+        let module = Deployer::with_preset("customsection")
+            .unwrap()
+            .translate(&payload)
+            .unwrap()
+            .unwrap();
+        let expected = FromHex::from_hex(
+            "
+            0061736d010000000113046000017f60037f7f7f0060027f7f00600000023e0308
+            657468657265756d0b676574436f646553697a65000008657468657265756d0863
+            6f6465436f7079000108657468657265756d0666696e6973680002030201030503
+            010001071102066d656d6f72790200046d61696e00030a2c012a01037f10002100
+            4100410020001001200041046b2802002102200041046b20026b21012001200210
+            020b0015086465706c6f7965720061736d0100000008000000
+        ",
+        )
+        .unwrap();
+        let output = module.to_bytes().unwrap();
+        assert_eq!(output, expected);
+    }
+
+    #[test]
+    fn memory_interface_test() {
+        let payload = Module::default();
+        let module = Deployer::with_preset("memory")
+            .unwrap()
+            .translate(&payload)
+            .unwrap()
+            .unwrap();
+        let expected = FromHex::from_hex(
+            "
+            0061736d0100000001090260027f7f0060000002130108657468657265756d0666
+            696e697368000003030200010503010001071102046d61696e0002066d656d6f72
+            7902000a0d0202000b08004100410810000b0b0e010041000b080061736d010000
+            00
+        ",
+        )
+        .unwrap();
+        let output = module.to_bytes().unwrap();
+        assert_eq!(output, expected);
+    }
 }
