@@ -1,17 +1,15 @@
-extern crate byteorder;
-extern crate parity_wasm;
-extern crate rustc_hex;
+pub use parity_wasm::elements::Module;
 
-use parity_wasm::elements::Module;
+use std::{error, fmt};
 
 pub mod imports;
 
+#[cfg(feature = "binaryen")]
+pub mod binaryenopt;
 pub mod checkfloat;
 pub mod checkstartfunc;
 pub mod deployer;
 pub mod dropsection;
-#[cfg(feature = "wabt")]
-pub mod fromwat;
 pub mod remapimports;
 pub mod remapstart;
 pub mod repack;
@@ -22,17 +20,33 @@ pub mod verifyexports;
 pub mod verifyimports;
 
 mod depgraph;
-mod utils;
-
-use std::{error, fmt};
 
 #[derive(Eq, PartialEq, Debug)]
+pub enum ModuleKind {
+    Creator,
+    Translator,
+    Validator,
+}
+
+#[derive(Eq, PartialEq, Debug, Clone)]
 pub enum ModuleError {
     NotSupported,
     NotFound,
     Custom(String),
 }
 
+/// Utility interface for chisel modules.
+pub trait ChiselModule<'a> {
+    type ObjectReference: ?Sized;
+    /// Returns the name of the chisel module.
+    fn id(&'a self) -> String;
+
+    fn kind(&'a self) -> ModuleKind;
+
+    /// Borrows the instance as a trait object.
+    fn as_abstract(&'a self) -> Self::ObjectReference;
+}
+
 pub trait ModuleCreator {
     /// Returns new module.
     fn create(&self) -> Result<Module, ModuleError>;
@@ -52,7 +66,7 @@ pub trait ModuleValidator {
 }
 
 pub trait ModulePreset {
-    fn with_preset(preset: &str) -> Result<Self, ()>
+    fn with_preset(preset: &str) -> Result<Self, ModuleError>
     where
         Self: std::marker::Sized;
 }
@@ -70,6 +84,14 @@ impl From<std::io::Error> for ModuleError {
     }
 }
 
+// Also aliased as parity_wasm::SerializationError
+impl From<parity_wasm::elements::Error> for ModuleError {
+    fn from(a: parity_wasm::elements::Error) -> Self {
+        use std::error::Error;
+        ModuleError::Custom(a.description().to_string())
+    }
+}
+
 impl fmt::Display for ModuleError {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         write!(
@@ -93,17 +115,17 @@ impl error::Error for ModuleError {
         }
     }
 
-    fn cause(&self) -> Option<&error::Error> {
+    fn cause(&self) -> Option<&dyn error::Error> {
         None
     }
 }
 
 #[cfg(test)]
 mod tests {
-    use super::*;
-
     use std::error::Error;
 
+    use super::*;
+
     struct SampleModule {}
 
     impl ModuleCreator for SampleModule {
@@ -127,6 +149,24 @@ mod tests {
         }
     }
 
+    impl<'a> ChiselModule<'a> for SampleModule {
+        // Yes, it implements all the traits, but we will treat it as a validator when used as a
+        // trait object for testing purposes.
+        type ObjectReference = &'a dyn ModuleValidator;
+
+        fn id(&'a self) -> String {
+            "Sample".to_string()
+        }
+
+        fn kind(&'a self) -> ModuleKind {
+            ModuleKind::Validator
+        }
+
+        fn as_abstract(&'a self) -> Self::ObjectReference {
+            self as Self::ObjectReference
+        }
+    }
+
     #[test]
     fn creator_succeeds() {
         let creator = SampleModule {};
@@ -182,4 +222,20 @@ mod tests {
         let err_description_custom = err_custom.description();
         assert_eq!("bar", err_description_custom);
     }
+
+    #[test]
+    fn opaque_module() {
+        let validator = SampleModule {};
+        assert_eq!(validator.id(), "Sample");
+
+        let opaque: &dyn ChiselModule<ObjectReference = &dyn ModuleValidator> =
+            &validator as &dyn ChiselModule<ObjectReference = &dyn ModuleValidator>;
+
+        assert_eq!(opaque.kind(), ModuleKind::Validator);
+
+        let as_trait: &dyn ModuleValidator = opaque.as_abstract();
+
+        let result = as_trait.validate(&Module::default());
+        assert!(result.is_ok());
+    }
 }
