@@ -1,6 +1,8 @@
-use super::{imports::ImportList, ModuleError, ModulePreset, ModuleTranslator};
+use parity_wasm::elements::{ImportEntry, ImportSection, Module};
 
-use parity_wasm::elements::*;
+use super::{
+    imports::ImportList, ChiselModule, ModuleError, ModuleKind, ModulePreset, ModuleTranslator,
+};
 
 pub struct RemapImports<'a> {
     /// A list of import sets to remap.
@@ -11,8 +13,24 @@ pub struct RemapImports<'a> {
 /// imports are expected to be prefixed.
 pub struct ImportInterface<'a>(ImportList<'a>, Option<&'a str>);
 
+impl<'a> ChiselModule<'a> for RemapImports<'a> {
+    type ObjectReference = &'a dyn ModuleTranslator;
+
+    fn id(&'a self) -> String {
+        "remapimports".to_string()
+    }
+
+    fn kind(&'a self) -> ModuleKind {
+        ModuleKind::Translator
+    }
+
+    fn as_abstract(&'a self) -> Self::ObjectReference {
+        self as Self::ObjectReference
+    }
+}
+
 impl<'a> ModulePreset for RemapImports<'a> {
-    fn with_preset(preset: &str) -> Result<Self, ()> {
+    fn with_preset(preset: &str) -> Result<Self, ModuleError> {
         let mut interface_set: Vec<ImportInterface> = Vec::new();
 
         // Accept a comma-separated list of presets.
@@ -23,22 +41,22 @@ impl<'a> ModulePreset for RemapImports<'a> {
         for preset_individual in presets.split(',') {
             match preset_individual {
                 "ewasm" => interface_set.push(ImportInterface::new(
-                    ImportList::with_preset("ewasm").expect("Missing ewasm preset"),
+                    ImportList::with_preset("ewasm")?,
                     Some("ethereum_"),
                 )),
                 "eth2" => interface_set.push(ImportInterface::new(
-                    ImportList::with_preset("eth2").expect("Missing eth2 preset"),
+                    ImportList::with_preset("eth2")?,
                     Some("eth2_"),
                 )),
                 "debug" => interface_set.push(ImportInterface::new(
-                    ImportList::with_preset("debug").expect("Missing debug preset"),
+                    ImportList::with_preset("debug")?,
                     Some("debug_"),
                 )),
                 "bignum" => interface_set.push(ImportInterface::new(
-                    ImportList::with_preset("bignum").expect("Missing bignum preset"),
+                    ImportList::with_preset("bignum")?,
                     Some("bignum_"),
                 )),
-                _ => return Err(()),
+                _ => return Err(ModuleError::NotSupported),
             }
         }
 
@@ -107,6 +125,8 @@ impl<'a> ImportInterface<'a> {
 }
 
 impl<'a> RemapImports<'a> {
+    // NOTE: 'new()' is currently unused in the library but useful in the future.
+    #[allow(dead_code)]
     fn new(interfaces: Vec<ImportInterface<'a>>) -> Self {
         RemapImports {
             interfaces: interfaces,
@@ -158,11 +178,11 @@ impl<'a> RemapImports<'a> {
 
 #[cfg(test)]
 mod tests {
+    use rustc_hex::FromHex;
+
     use super::*;
     use crate::verifyimports::*;
     use crate::{ModulePreset, ModuleTranslator, ModuleValidator};
-    use parity_wasm;
-    use rustc_hex::FromHex;
 
     #[test]
     fn smoke_test() {
@@ -173,12 +193,12 @@ mod tests {
         ",
         )
         .unwrap();
-        let mut module = parity_wasm::deserialize_buffer(&input).expect("failed");
+        let mut module = Module::from_bytes(&input).unwrap();
         let did_change = RemapImports::with_preset("ewasm")
             .unwrap()
             .translate_inplace(&mut module)
             .unwrap();
-        let output = parity_wasm::serialize(module).expect("failed");
+        let output = module.to_bytes().unwrap();
         let expected = FromHex::from_hex(
             "
             0061736d0100000001050160017e0002130108657468657265756d067573
@@ -210,7 +230,7 @@ mod tests {
             0x00, 0x0a, 0x04, 0x01, 0x02, 0x00, 0x0b,
         ];
 
-        let module = parity_wasm::deserialize_buffer(&wasm).unwrap();
+        let module = Module::from_bytes(&wasm).unwrap();
 
         let new = RemapImports::with_preset("ewasm")
             .unwrap()
@@ -240,7 +260,7 @@ mod tests {
             0x00, 0x0a, 0x04, 0x01, 0x02, 0x00, 0x0b,
         ];
 
-        let module = parity_wasm::deserialize_buffer(&wasm).unwrap();
+        let module = Module::from_bytes(&wasm).unwrap();
 
         let new = RemapImports::with_preset("ewasm")
             .unwrap()
@@ -278,7 +298,7 @@ mod tests {
             0x6f, 0x72, 0x79, 0x02, 0x00, 0x0a, 0x04, 0x01, 0x02, 0x00, 0x0b,
         ];
 
-        let module = parity_wasm::deserialize_buffer(&wasm).unwrap();
+        let module = Module::from_bytes(&wasm).unwrap();
 
         let new = RemapImports::with_preset("ewasm")
             .unwrap()
@@ -324,7 +344,7 @@ mod tests {
             0x72, 0x79, 0x02, 0x00, 0x0a, 0x04, 0x01, 0x02, 0x00, 0x0b,
         ];
 
-        let module = parity_wasm::deserialize_buffer(&wasm).unwrap();
+        let module = Module::from_bytes(&wasm).unwrap();
 
         let new = RemapImports::with_preset("ewasm, bignum, debug")
             .unwrap()
@@ -365,7 +385,7 @@ mod tests {
             0x00, 0x0b,
         ];
 
-        let module = parity_wasm::deserialize_buffer(&wasm).unwrap();
+        let module = Module::from_bytes(&wasm).unwrap();
 
         let interfaces_noprefix = vec![
             ImportInterface::new(ImportList::with_preset("ewasm").unwrap(), None),
