@@ -1,6 +1,7 @@
-use super::{ModuleError, ModuleTranslator};
 use parity_wasm::builder;
-use parity_wasm::elements::*;
+use parity_wasm::elements::Module;
+
+use super::{ChiselModule, ModuleError, ModuleKind, ModuleTranslator};
 
 pub struct Repack;
 
@@ -10,6 +11,22 @@ impl Repack {
     }
 }
 
+impl<'a> ChiselModule<'a> for Repack {
+    type ObjectReference = &'a dyn ModuleTranslator;
+
+    fn id(&'a self) -> String {
+        "repack".to_string()
+    }
+
+    fn kind(&'a self) -> ModuleKind {
+        ModuleKind::Translator
+    }
+
+    fn as_abstract(&'a self) -> Self::ObjectReference {
+        self as Self::ObjectReference
+    }
+}
+
 impl ModuleTranslator for Repack {
     fn translate_inplace(&self, _module: &mut Module) -> Result<bool, ModuleError> {
         Err(ModuleError::NotSupported)
@@ -25,11 +42,12 @@ impl ModuleTranslator for Repack {
 
 #[cfg(test)]
 mod tests {
-    use super::*;
-    use crate::utils::*;
     use parity_wasm::builder;
+    use parity_wasm::elements::CustomSection;
     use rustc_hex::FromHex;
 
+    use super::*;
+
     #[test]
     fn smoke_test() {
         let module = Module::default();
@@ -101,7 +119,7 @@ mod tests {
 61696e",
         )
         .unwrap();
-        let module = Module::from_slice(&input);
+        let module = Module::from_bytes(&input).unwrap();
         // Forcefully parse names section here.
         let module = module
             .parse_names()
