@@ -1,5 +1,6 @@
-use super::{ModuleError, ModulePreset, ModuleTranslator};
-use parity_wasm::elements::*;
+use parity_wasm::elements::{ExportEntry, ExportSection, Internal, Module};
+
+use super::{ChiselModule, ModuleError, ModuleKind, ModulePreset, ModuleTranslator};
 
 /// Struct containing a list of valid exports.
 struct ExportWhitelist {
@@ -12,13 +13,29 @@ pub struct TrimExports {
     whitelist: ExportWhitelist,
 }
 
+impl<'a> ChiselModule<'a> for TrimExports {
+    type ObjectReference = &'a dyn ModuleTranslator;
+
+    fn id(&'a self) -> String {
+        "trimexports".to_string()
+    }
+
+    fn kind(&'a self) -> ModuleKind {
+        ModuleKind::Translator
+    }
+
+    fn as_abstract(&'a self) -> Self::ObjectReference {
+        self as Self::ObjectReference
+    }
+}
+
 /// Helper that compares the enum variant of two WebAssembly exports.
 fn cmp_internal_variant(a: &Internal, b: &Internal) -> bool {
     std::mem::discriminant(a) == std::mem::discriminant(b)
 }
 
 impl ModulePreset for ExportWhitelist {
-    fn with_preset(preset: &str) -> Result<Self, ()> {
+    fn with_preset(preset: &str) -> Result<Self, ModuleError> {
         match preset {
             "ewasm" => Ok(ExportWhitelist {
                 entries: vec![
@@ -30,7 +47,7 @@ impl ModulePreset for ExportWhitelist {
             "pwasm" => Ok(ExportWhitelist {
                 entries: vec![ExportEntry::new("_call".to_string(), Internal::Function(0))],
             }),
-            _ => Err(()),
+            _ => Err(ModuleError::NotSupported),
         }
     }
 }
@@ -63,20 +80,6 @@ impl TrimExports {
         }
     }
 
-    /// Takes a given preset string and constructs a context with the
-    /// corresponding whitelist.
-    pub fn with_preset(preset: &str) -> Result<Self, ()> {
-        match preset {
-            "ewasm" => Ok(TrimExports {
-                whitelist: ExportWhitelist::with_preset("ewasm").unwrap(),
-            }),
-            "pwasm" => Ok(TrimExports {
-                whitelist: ExportWhitelist::with_preset("pwasm").unwrap(),
-            }),
-            _ => Err(()),
-        }
-    }
-
     /// Iterates over the export section, if there is one, and removes
     /// unnecessary entries.
     fn trim_exports(&self, module: &mut Module) -> bool {
@@ -102,6 +105,22 @@ impl TrimExports {
     }
 }
 
+impl ModulePreset for TrimExports {
+    /// Takes a given preset string and constructs a context with the
+    /// corresponding whitelist.
+    fn with_preset(preset: &str) -> Result<Self, ModuleError> {
+        match preset {
+            "ewasm" => Ok(TrimExports {
+                whitelist: ExportWhitelist::with_preset("ewasm")?,
+            }),
+            "pwasm" => Ok(TrimExports {
+                whitelist: ExportWhitelist::with_preset("pwasm")?,
+            }),
+            _ => Err(ModuleError::NotSupported),
+        }
+    }
+}
+
 impl ModuleTranslator for TrimExports {
     fn translate_inplace(&self, module: &mut Module) -> Result<bool, ModuleError> {
         Ok(self.trim_exports(module))
@@ -119,9 +138,10 @@ impl ModuleTranslator for TrimExports {
 
 #[cfg(test)]
 mod tests {
-    use super::*;
     use parity_wasm::builder;
 
+    use super::*;
+
     // Smoke tests
     #[test]
     fn builder_all_exports_good_ewasm() {
