@@ -1,5 +1,6 @@
-use super::{ModuleError, ModulePreset, ModuleTranslator};
-use parity_wasm::elements::*;
+use parity_wasm::elements::Module;
+
+use super::{ChiselModule, ModuleError, ModuleKind, ModulePreset, ModuleTranslator};
 
 pub struct TrimStartFunc;
 
@@ -14,11 +15,27 @@ impl TrimStartFunc {
     }
 }
 
+impl<'a> ChiselModule<'a> for TrimStartFunc {
+    type ObjectReference = &'a dyn ModuleTranslator;
+
+    fn id(&'a self) -> String {
+        "trimstartfunc".to_string()
+    }
+
+    fn kind(&'a self) -> ModuleKind {
+        ModuleKind::Translator
+    }
+
+    fn as_abstract(&'a self) -> Self::ObjectReference {
+        self as Self::ObjectReference
+    }
+}
+
 impl ModulePreset for TrimStartFunc {
-    fn with_preset(preset: &str) -> Result<Self, ()> {
+    fn with_preset(preset: &str) -> Result<Self, ModuleError> {
         match preset {
             "ewasm" => Ok(TrimStartFunc {}),
-            _ => Err(()),
+            _ => Err(ModuleError::NotSupported),
         }
     }
 }
@@ -28,16 +45,19 @@ impl ModuleTranslator for TrimStartFunc {
         Ok(self.trim_startfunc(module))
     }
 
-    fn translate(&self, _module: &Module) -> Result<Option<Module>, ModuleError> {
-        Err(ModuleError::NotSupported)
+    fn translate(&self, module: &Module) -> Result<Option<Module>, ModuleError> {
+        let mut ret = module.clone();
+        let modified = self.trim_startfunc(&mut ret);
+        if modified {
+            return Ok(Some(ret));
+        }
+        Ok(None)
     }
 }
 
 #[cfg(test)]
 mod tests {
-
     use super::*;
-    use parity_wasm::elements::deserialize_buffer;
 
     #[test]
     fn start_removed() {
@@ -47,12 +67,12 @@ mod tests {
             0x08, 0x01, 0x00, 0x0a, 0x04, 0x01, 0x02, 0x00, 0x0b,
         ];
 
-        let mut module = deserialize_buffer::<Module>(&wasm).unwrap();
+        let mut module = Module::from_bytes(&wasm).unwrap();
 
         let trimmer = TrimStartFunc::with_preset("ewasm").unwrap();
         trimmer.translate_inplace(&mut module).unwrap();
 
-        let result = serialize::<Module>(module).unwrap();
+        let result = module.to_bytes().unwrap();
         let expect: Vec<u8> = vec![
             0x00, 0x61, 0x73, 0x6d, 0x01, 0x00, 0x00, 0x00, 0x01, 0x04, 0x01, 0x60, 0x00, 0x00,
             0x03, 0x02, 0x01, 0x00, 0x07, 0x08, 0x01, 0x04, 0x6d, 0x61, 0x69, 0x6e, 0x00, 0x00,
@@ -70,12 +90,12 @@ mod tests {
             0x0a, 0x04, 0x01, 0x02, 0x00, 0x0b,
         ];
 
-        let mut module = deserialize_buffer::<Module>(&wasm).unwrap();
+        let mut module = Module::from_bytes(&wasm).unwrap();
 
         let trimmer = TrimStartFunc::with_preset("ewasm").unwrap();
         trimmer.translate_inplace(&mut module).unwrap();
 
-        let result = serialize::<Module>(module).unwrap();
+        let result = module.to_bytes().unwrap();
 
         // result is equal to initial wasm (not changed)
         assert_eq!(result, wasm);
