@@ -1,8 +1,9 @@
-use super::{ModuleError, ModulePreset, ModuleValidator};
 use parity_wasm::elements::{
     ExportSection, External, FunctionSection, FunctionType, ImportSection, Internal, Module, Type,
 };
 
+use super::{ChiselModule, ModuleError, ModuleKind, ModulePreset, ModuleValidator};
+
 /// Enum representing a type of export and any extra data to check.
 pub enum ExportType<'a> {
     Function(&'a str, FunctionType),
@@ -22,8 +23,24 @@ pub struct VerifyExports<'a> {
     allow_unlisted: bool,
 }
 
+impl<'a> ChiselModule<'a> for VerifyExports<'a> {
+    type ObjectReference = &'a dyn ModuleValidator;
+
+    fn id(&'a self) -> String {
+        "verifyexports".to_string()
+    }
+
+    fn kind(&'a self) -> ModuleKind {
+        ModuleKind::Validator
+    }
+
+    fn as_abstract(&'a self) -> Self::ObjectReference {
+        self as Self::ObjectReference
+    }
+}
+
 impl<'a> ModulePreset for VerifyExports<'a> {
-    fn with_preset(preset: &str) -> Result<Self, ()> {
+    fn with_preset(preset: &str) -> Result<Self, ModuleError> {
         match preset {
             "ewasm" => Ok(VerifyExports {
                 entries: vec![
@@ -32,7 +49,7 @@ impl<'a> ModulePreset for VerifyExports<'a> {
                 ],
                 allow_unlisted: false,
             }),
-            _ => Err(()),
+            _ => Err(ModuleError::NotSupported),
         }
     }
 }
@@ -152,13 +169,11 @@ fn func_sig_by_index(module: &Module, index: u32) -> Option<&FunctionType> {
                 index - func_import_section_len(import_section),
             )] {
                 Type::Function(ref ret) => Some(ret),
-                _ => None,
             },
             // If no function imports are present, no need to subtract them.
             (Some(type_section), None) => {
                 match type_section.types()[func_type_ref(&func_section, index)] {
                     Type::Function(ref ret) => Some(ret),
-                    _ => None,
                 }
             }
             (None, Some(_import_section)) => None,
@@ -202,7 +217,6 @@ fn func_export_index_by_name(exports: &ExportSection, field_str: &str) -> Option
 #[cfg(test)]
 mod tests {
     use super::*;
-    use parity_wasm::elements::deserialize_buffer;
 
     #[test]
     fn no_exports() {
@@ -210,7 +224,7 @@ mod tests {
         // (module)
         let wasm: Vec<u8> = vec![0x00, 0x61, 0x73, 0x6d, 0x01, 0x00, 0x00, 0x00];
 
-        let module = deserialize_buffer::<Module>(&wasm).unwrap();
+        let module = Module::from_bytes(&wasm).unwrap();
         let checker = VerifyExports::with_preset("ewasm").unwrap();
         let result = checker.validate(&module).unwrap();
         assert_eq!(false, result);
@@ -232,7 +246,7 @@ mod tests {
             0x0a, 0x04, 0x01, 0x02, 0x00, 0x0b,
         ];
 
-        let module = deserialize_buffer::<Module>(&wasm).unwrap();
+        let module = Module::from_bytes(&wasm).unwrap();
         let checker = VerifyExports::with_preset("ewasm").unwrap();
         let result = checker.validate(&module).unwrap();
         assert_eq!(true, result);
@@ -256,7 +270,7 @@ mod tests {
             0x00, 0x0a, 0x06, 0x01, 0x04, 0x00, 0x41, 0x00, 0x0b,
         ];
 
-        let module = deserialize_buffer::<Module>(&wasm).unwrap();
+        let module = Module::from_bytes(&wasm).unwrap();
         let checker = VerifyExports::with_preset("ewasm").unwrap();
         let result = checker.validate(&module).unwrap();
         assert_eq!(false, result);
@@ -278,7 +292,7 @@ mod tests {
             0x00, 0x0a, 0x04, 0x01, 0x02, 0x00, 0x0b,
         ];
 
-        let module = deserialize_buffer::<Module>(&wasm).unwrap();
+        let module = Module::from_bytes(&wasm).unwrap();
         let checker = VerifyExports::with_preset("ewasm").unwrap();
         let result = checker.validate(&module).unwrap();
         assert_eq!(false, result);
@@ -298,7 +312,7 @@ mod tests {
             0x6d, 0x61, 0x69, 0x6e, 0x00, 0x00, 0x0a, 0x04, 0x01, 0x02, 0x00, 0x0b,
         ];
 
-        let module = deserialize_buffer::<Module>(&wasm).unwrap();
+        let module = Module::from_bytes(&wasm).unwrap();
         let checker = VerifyExports::with_preset("ewasm").unwrap();
         let result = checker.validate(&module).unwrap();
         assert_eq!(false, result);
@@ -318,7 +332,7 @@ mod tests {
             0x6d, 0x65, 0x6d, 0x6f, 0x72, 0x79, 0x02, 0x00, 0x0a, 0x04, 0x01, 0x02, 0x00, 0x0b,
         ];
 
-        let module = deserialize_buffer::<Module>(&wasm).unwrap();
+        let module = Module::from_bytes(&wasm).unwrap();
         let checker = VerifyExports::with_preset("ewasm").unwrap();
         let result = checker.validate(&module).unwrap();
         assert_eq!(false, result);
@@ -340,7 +354,7 @@ mod tests {
             0x0a, 0x04, 0x01, 0x02, 0x00, 0x0b,
         ];
 
-        let module = deserialize_buffer::<Module>(&wasm).unwrap();
+        let module = Module::from_bytes(&wasm).unwrap();
         let checker = VerifyExports::with_preset("ewasm").unwrap();
         let result = checker.validate(&module).unwrap();
         assert_eq!(false, result);
@@ -362,7 +376,7 @@ mod tests {
             0x0a, 0x04, 0x01, 0x02, 0x00, 0x0b,
         ];
 
-        let module = deserialize_buffer::<Module>(&wasm).unwrap();
+        let module = Module::from_bytes(&wasm).unwrap();
         let checker = VerifyExports::with_preset("ewasm").unwrap();
         let result = checker.validate(&module).unwrap();
         assert_eq!(false, result);
@@ -384,7 +398,7 @@ mod tests {
             0x04, 0x01, 0x02, 0x00, 0x0b,
         ];
 
-        let module = deserialize_buffer::<Module>(&wasm).unwrap();
+        let module = Module::from_bytes(&wasm).unwrap();
         let checker = VerifyExports::with_preset("ewasm").unwrap();
         let result = checker.validate(&module).unwrap();
         assert_eq!(false, result);
@@ -408,7 +422,7 @@ mod tests {
             0x0b,
         ];
 
-        let module = deserialize_buffer::<Module>(&wasm).unwrap();
+        let module = Module::from_bytes(&wasm).unwrap();
         let checker = VerifyExports::with_preset("ewasm").unwrap();
         let result = checker.validate(&module).unwrap();
         assert_eq!(false, result);
@@ -432,7 +446,7 @@ mod tests {
             0x0b,
         ];
 
-        let module = deserialize_buffer::<Module>(&wasm).unwrap();
+        let module = Module::from_bytes(&wasm).unwrap();
         let checker = VerifyExports {
             entries: vec![
                 ExportType::Function("main", FunctionType::default()),
