@@ -1,8 +1,9 @@
+use parity_wasm::elements::{External, FunctionType, ImportSection, Module, Type};
+
 use super::{
     imports::{ImportList, ImportType},
-    ModuleError, ModulePreset, ModuleValidator,
+    ChiselModule, ModuleError, ModuleKind, ModulePreset, ModuleValidator,
 };
-use parity_wasm::elements::{External, FunctionType, ImportSection, Module, Type};
 
 /// Enum representing the state of an import in a module.
 #[derive(PartialEq)]
@@ -34,8 +35,24 @@ pub struct VerifyImports<'a> {
     allow_unlisted: bool,
 }
 
+impl<'a> ChiselModule<'a> for VerifyImports<'a> {
+    type ObjectReference = &'a dyn ModuleValidator;
+
+    fn id(&'a self) -> String {
+        "verifyimports".to_string()
+    }
+
+    fn kind(&'a self) -> ModuleKind {
+        ModuleKind::Validator
+    }
+
+    fn as_abstract(&'a self) -> Self::ObjectReference {
+        self as Self::ObjectReference
+    }
+}
+
 impl<'a> ModulePreset for VerifyImports<'a> {
-    fn with_preset(preset: &str) -> Result<Self, ()> {
+    fn with_preset(preset: &str) -> Result<Self, ModuleError> {
         let mut import_set = ImportList::new();
         let presets: String = preset
             .chars()
@@ -43,7 +60,7 @@ impl<'a> ModulePreset for VerifyImports<'a> {
             .collect();
 
         for preset_individual in presets.split(',') {
-            let to_append = ImportList::with_preset(preset_individual).expect("Invalid preset");
+            let to_append = ImportList::with_preset(preset_individual)?;
             import_set.concatenate(to_append);
         }
 
@@ -287,8 +304,9 @@ pub fn imported_func_sig_by_index(module: &Module, index: usize) -> FunctionType
 
 #[cfg(test)]
 mod tests {
+    use parity_wasm::elements::ValueType;
+
     use super::*;
-    use parity_wasm::elements::{deserialize_buffer, ValueType};
 
     #[test]
     fn no_imports_ok_ewasm() {
@@ -306,7 +324,7 @@ mod tests {
             0x0a, 0x04, 0x01, 0x02, 0x00, 0x0b,
         ];
 
-        let module = deserialize_buffer::<Module>(&wasm).unwrap();
+        let module = Module::from_bytes(&wasm).unwrap();
         let checker = VerifyImports::with_preset("ewasm").unwrap();
         let result = checker.validate(&module).unwrap();
         assert_eq!(true, result);
@@ -331,7 +349,7 @@ mod tests {
             0x72, 0x79, 0x02, 0x00, 0x0a, 0x04, 0x01, 0x02, 0x00, 0x0b,
         ];
 
-        let module = deserialize_buffer::<Module>(&wasm).unwrap();
+        let module = Module::from_bytes(&wasm).unwrap();
         let checker = VerifyImports::with_preset("ewasm").unwrap();
         let result = checker.validate(&module).unwrap();
         assert_eq!(true, result);
@@ -356,7 +374,7 @@ mod tests {
             0x79, 0x02, 0x00, 0x0a, 0x04, 0x01, 0x02, 0x00, 0x0b,
         ];
 
-        let module = deserialize_buffer::<Module>(&wasm).unwrap();
+        let module = Module::from_bytes(&wasm).unwrap();
         let checker = VerifyImports::with_preset("ewasm").unwrap();
         let result = checker.validate(&module).unwrap();
         assert_eq!(false, result);
@@ -381,7 +399,7 @@ mod tests {
             0x04, 0x01, 0x02, 0x00, 0x0b,
         ];
 
-        let module = deserialize_buffer::<Module>(&wasm).unwrap();
+        let module = Module::from_bytes(&wasm).unwrap();
         let checker = VerifyImports::with_preset("ewasm").unwrap();
         let result = checker.validate(&module).unwrap();
         assert_eq!(false, result);
@@ -406,7 +424,7 @@ mod tests {
             0x79, 0x02, 0x00, 0x0a, 0x04, 0x01, 0x02, 0x00, 0x0b,
         ];
 
-        let module = deserialize_buffer::<Module>(&wasm).unwrap();
+        let module = Module::from_bytes(&wasm).unwrap();
         let checker = VerifyImports::with_preset("ewasm").unwrap();
         let result = checker.validate(&module).unwrap();
         assert_eq!(false, result);
@@ -442,7 +460,7 @@ mod tests {
             0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
         ];
 
-        let module = deserialize_buffer::<Module>(&wasm).unwrap();
+        let module = Module::from_bytes(&wasm).unwrap();
         let checker = VerifyImports::with_preset("ewasm").unwrap();
         let result = checker.validate(&module).unwrap();
         assert_eq!(true, result);
@@ -469,7 +487,7 @@ mod tests {
             0x6d, 0x65, 0x6d, 0x6f, 0x72, 0x79, 0x02, 0x00, 0x0a, 0x04, 0x01, 0x02, 0x00, 0x0b,
         ];
 
-        let module = deserialize_buffer::<Module>(&wasm).unwrap();
+        let module = Module::from_bytes(&wasm).unwrap();
         let checker = VerifyImports::with_preset("ewasm").unwrap();
         let result = checker.validate(&module).unwrap();
         assert_eq!(false, result);
@@ -496,7 +514,7 @@ mod tests {
             0x6d, 0x65, 0x6d, 0x6f, 0x72, 0x79, 0x02, 0x00, 0x0a, 0x04, 0x01, 0x02, 0x00, 0x0b,
         ];
 
-        let module = deserialize_buffer::<Module>(&wasm).unwrap();
+        let module = Module::from_bytes(&wasm).unwrap();
         let mut checker = VerifyImports::with_preset("ewasm").unwrap();
         // Allow unlisted, just for this test case
         checker.set_allow_unlisted(true);
@@ -523,7 +541,7 @@ mod tests {
             0x72, 0x79, 0x02, 0x00, 0x0a, 0x04, 0x01, 0x02, 0x00, 0x0b,
         ];
 
-        let module = deserialize_buffer::<Module>(&wasm).unwrap();
+        let module = Module::from_bytes(&wasm).unwrap();
         let mut checker = VerifyImports::with_preset("ewasm").unwrap();
         // Require all, just for this test case
         checker.set_require_all(true);
@@ -550,7 +568,7 @@ mod tests {
             0x72, 0x79, 0x02, 0x00, 0x0a, 0x04, 0x01, 0x02, 0x00, 0x0b,
         ];
 
-        let module = deserialize_buffer::<Module>(&wasm).unwrap();
+        let module = Module::from_bytes(&wasm).unwrap();
         let checker = VerifyImports {
             list: ImportList::with_entries(vec![ImportType::Function(
                 "ethereum",
@@ -585,7 +603,7 @@ mod tests {
             0x6f, 0x72, 0x79, 0x02, 0x00, 0x0a, 0x04, 0x01, 0x02, 0x00, 0x0b,
         ];
 
-        let module = deserialize_buffer::<Module>(&wasm).unwrap();
+        let module = Module::from_bytes(&wasm).unwrap();
         let checker = VerifyImports {
             list: ImportList::with_entries(vec![ImportType::Function(
                 "ethereum",
@@ -620,7 +638,7 @@ mod tests {
             0x6f, 0x72, 0x79, 0x02, 0x00, 0x0a, 0x04, 0x01, 0x02, 0x00, 0x0b,
         ];
 
-        let module = deserialize_buffer::<Module>(&wasm).unwrap();
+        let module = Module::from_bytes(&wasm).unwrap();
         let checker = VerifyImports {
             list: ImportList::with_preset("ewasm").unwrap(),
             allow_unlisted: false,
@@ -651,7 +669,7 @@ mod tests {
             0x0a, 0x04, 0x01, 0x02, 0x00, 0x0b,
         ];
 
-        let module = deserialize_buffer::<Module>(&wasm).unwrap();
+        let module = Module::from_bytes(&wasm).unwrap();
         let checker = VerifyImports {
             list: ImportList::with_preset("ewasm").unwrap(),
             allow_unlisted: false,
@@ -688,7 +706,7 @@ mod tests {
             0x69, 0x6e, 0x00, 0x01, 0x0a, 0x04, 0x01, 0x02, 0x00, 0x0b,
         ];
 
-        let module = deserialize_buffer::<Module>(&wasm).unwrap();
+        let module = Module::from_bytes(&wasm).unwrap();
         let checker = VerifyImports::with_preset("ewasm").unwrap();
 
         let result = checker.validate(&module).unwrap();
